
/* Custom styling for tool box */
toolitemgroup > button {
    border-width: 1px 0px 0px 0px;
    border-radius: 0px;
    padding-top: 3px;
    padding-bottom: 3px;
    background-image: none;
    box-shadow: none;
}

toolitemgroup:first-child > button {
    border-width: 0px;
}

#diagrams {
    border-top-width: 0px;
}

diagramview {
    background: white;
}

@binding-set expand-collapse-row {
  bind "Left" { "expand-collapse-cursor-row" (1, 0, 0) };
  bind "Right" { "expand-collapse-cursor-row" (1, 1, 0) };
}

treeview {
  -gtk-key-bindings: expand-collapse-row;
}